#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Drawing;

namespace HeuristicLab.Optimization.Networks {
  [Item("Network", "An optimization network.")]
  [Creatable("Optimization Networks")]
  [StorableClass]
  public class Network : Node, INetwork, IStorableContent {
    public string Filename { get; set; }

    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Module; }
    }

    [Storable]
    private NodeCollection nodes;
    public NodeCollection Nodes {
      get { return nodes; }
    }

    [StorableConstructor]
    protected Network(bool deserializing) : base(deserializing) { }
    protected Network(Network original, Cloner cloner)
      : base(original, cloner) {
      nodes = cloner.Clone(original.nodes);
    }
    public Network()
      : base("Network") {
      nodes = new NodeCollection();
    }
    public Network(string name)
      : base(name) {
      nodes = new NodeCollection();
    }
    public Network(string name, string description)
      : base(name, description) {
      nodes = new NodeCollection();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Network(this, cloner);
    }
  }
}
