#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Drawing;

namespace HeuristicLab.Optimization.Networks {
  [Item("Port", "Abstract base class for ports of an optimization network node.")]
  [StorableClass]
  public abstract class Port : NamedItem, IPort {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Field; }
    }

    private INode parent;
    public INode Parent {
      get { return parent; }
      set {
        // NOTE: never call setter directly as the Parent property is set by the node which contains the port
        if (value != parent) {
          DeregisterParentEvents();
          parent = value;
          RegisterParentEvents();
          OnPathChanged();
        }
      }
    }
    public string Path {
      get {
        return parent != null ? parent.Name : string.Empty;
      }
    }

    [StorableConstructor]
    protected Port(bool deserializing) : base(deserializing) { }
    protected Port(Port original, Cloner cloner) : base(original, cloner) { }
    protected Port() : base("Port") { }
    protected Port(string name) : base(name) { }
    protected Port(string name, string description) : base(name, description) { }

    public event EventHandler PathChanged;
    protected virtual void OnPathChanged() {
      var handler = PathChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    #region Parent Events
    protected virtual void RegisterParentEvents() {
      if (parent != null) {
        parent.NameChanged += Parent_NameChanged;
      }
    }
    protected virtual void DeregisterParentEvents() {
      if (parent != null) {
        parent.NameChanged -= Parent_NameChanged;
      }
    }
    void Parent_NameChanged(object sender, EventArgs e) {
      OnPathChanged();
    }
    #endregion
  }
}
